/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 12.5 (4 Moments discrete), p. 325 
 *
 * this file: algo12_05.c
 *
 * For a demo how to use this algorithm see the file "demo_algo12_06.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

static double x0,x1,x2,x3,p0,p1,p2;

int setup12_05(double m1,double m2,double m3,double m4)
/*
 * Algorithm 12.5 (4 Moments discrete), p. 325 
 *
 * m1, m2, m3, m4 .... array holding the first four moments 
 */
{ 
       double m3t,m4t, s,q,p,p3;

/*2*/  if(m2-m1*m1<=0.){
         printf("second moment not feasible\n");
/*3*/	 return(0);
       }
/*4*/  s = sqrt(m2-m1*m1);
/*5*/  m3t =(m3-3*m1*m1+2.*m1*m1*m1)/(s*s*s);
       m4t=(m4-4.*m3*m1+6.*m2*m1*m1-3.*m1*m1*m1*m1)/(s*s*s*s);
/*6*/  if(m4t-1<=0.){
         printf("fourth moment not feasible\n");
/*7*/    return(0);
       }
/*8*/  q = 0.5*(1.+sqrt((m4t-1.)/(m4t+3)));
       p = 0.5*(1.+m3t/sqrt(m4t-1.));
/*9*/  if(p<0.||p>1.){
         printf("third moment not feasible\n");
/*10*/   return(0);
       }
/*11*/ x0 = m1+s*q/sqrt(q*(1.-q));
       p0 = p*(1.-q);
/*12*/ x1 = m1-s*(1.-q)/sqrt(q*(1.-q));
       p1 = p*q;
/*13*/ x2 = m1+s*(1.-q)/sqrt(q*(1.-q));
       p2 = (1.-p)*q;
/*14*/ x3 = m1-s*q/sqrt(q*(1.-q));
       p3 = (1.-p)*(1.-q);
       return 1;
}

 double generator12_05()
/* generates a random variate with the desired moments */
{
       double u;
/*15*/ u = uniform();
/*16*/ if(u<=p0) return x0;
/*17*/ else if(u<=p0+p1) return x1;
/*18*/ else if(u<=p0+p1+p2) return x2;
/*19*/ return x3;
}



