/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 4.4 (TDR-3-Points (for c = -1/2)), p. 94  
 *
 * this file: demo_algo04_04.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo04_04 demo_algo04_04.c algo04_04.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution and its derivative (dpdf)
   you can insert your own distribution here
*/

double my_pdf(double x)
{
  if( x >= 0. && x <= 1.)
    return 2.*x;
  return 0.;
}
/* triangular on (0,1) */





#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x,cp[5];

 setseed(13324821); /*set seed for uniform generator*/

 printf("Demo for Algorithm 4.4\nfirst we display ten numbers generated from my_pdf()\n");
 cp[0] = 0.1;
 cp[1] = 0.3;
 cp[2] = 0.5;
 cp[3] = 0.7;
 cp[4] = 0.9;
 setup04_04(my_pdf,0.,1.,1.);/* calls set-up*/
 for(i=0;i<10;i++){
   x = generator04_04();
      /* it is not necessary to specify the inputs again */
   printf("x %f\n",x);
 }

 printf("\nNow we make a chi-square test with sample size %d for  the normal distribution\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;
 cp[0] = -3.;
 cp[1] = -1.;
 cp[2] = 0.01;
/* note: the mode (0) is not allowed for this simple 
   implementation as this would lead to divison by zero */
 cp[3] = 1.;
 cp[4] = 3.;
 setup04_04(pdf_normal,-10.,0.,10.);/* calls set-up*/
 for(i=0;i<CLAS*CLAS;i++){
   x = generator04_04();
   obs[(int)(cdf_normal(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

