/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 11.10 (Uniform-Simplex, p. 257)
 *
 * this file: demo_algo11_10.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo11_10 demo_algo11_10.c algo11_10.c algo11_09.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


#define CLAS 500

static long int obs[CLAS];

int main()
{ int i,j;
 double x[10], xhelp, v[20]={0.,0.,0.,0.,
                      0.,.2,.2,.2,
                      0.,0.,2.,1.,
                      0.,1.,-1.,1.,
                      0.,1.,2.,1.},
                vv[12]={0.,0.,0.,
                      0.,0.,0.,
                      0.,2.,0.,
		       0.,2.,1.};

 setseed(13324821); /*set seed for uniform generator*/

 printf("Demo for Algorithm 11.10\n");
 printf("first we display ten generated vectors for dimension 3 and the vertices:\n");
 for(i=1;i<=4;i++){
   printf("%d. vertex:",i);
   for(j=1;j<=3;j++) printf("%5.1f ",v[i*(3+1)+j]);
   printf("\n");
 }
 for(i=0;i<10;i++){
   generator11_10(3,v,x);
   for(j=1;j<3;j++) printf("%8.3f, ",x[j]);
   printf("%8.3f\n",x[3]);
 }


 printf("Now we test the distributions for dimension 2 and vertices\n");
 for(i=1;i<=3;i++){
   printf("%d. vertex:",i);
   for(j=1;j<=2;j++) printf("%5.1f ",vv[i*(2+1)+j]);
   printf("\n");
 }

 for(i=0;i<CLAS;i++)obs[i]=0;
 printf("we test if 2.x[2]/x[1] is really uniform\n");
 for(i=0;i<CLAS*CLAS;i++){
   generator11_10(2,vv,x);
   xhelp = 2.*x[2]/x[1];
   obs[(int)((xhelp)*CLAS)]++;
   /*to test if 2.*x[2]/x[1] is uniform*/
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 

 return 0;
}


   

